/* Virtuino Iot example: ESP8266 Read/Write to digital pins
 * Supported boards: all ESP8266 boards
 * 
 * This example contains the following features
 * 1. How to read enable an Arduino led from Virtuino dashboard
 * 2. How to send the state of an arduino pin to Virtuino dashboard
 
 * Created by Ilias Lamprou
 * Updated Mar 10 2022
 */
 
#include <ESP8266WiFi.h>
#include <WebSocketsServer.h> // Download the library WebSockets by Markus Sattler from arduino library manager (ver 2.3.5)

//--- SETTINGS ----------------------------------------
  const char* ssid = "WiFi Name";                      // WIFI network SSID
  const char* password = "WiFi Password";              // WIFI network PASSWORD
  WebSocketsServer webSocket = WebSocketsServer(8000); // Default Virtuino Server port = 8000 
  //IPAddress ip(192, 168, 1, 150);                    // where 150 is the desired IP Address. The first three numbers must be the same as the router IP
  //IPAddress gateway(192, 168, 1, 1);                 // set gateway to match your network. Replace with your router IP

//---- Enter all the Tags here. You have to use the same Tags on Virtuino variables
  const char* pin0_tag= "V0";         // tag for digital input
  const char* pin1_tag= "V1";         // tag for digital input
  const char* pin5_tag= "V5";         // tag for digital output
  
  // add more here like the next lines... 
  // const char* pin6_tag= "V6";
  // const char* pin8_tag= "V8";

//---- Enter some variables to hold the last state of the inputs.
  uint8_t pin0_lastValue=0;
  uint8_t pin1_lastValue=0;
  uint8_t pin5_lastValue=0;
  
  // add more here like the next lines...
  // uint8_t pin6_lastValue=0;
  // uint8_t pin8_lastValue=0;


//===================================================== sendPinStatus
// It is called every time a new client is connected.
// The void informs Virtuino app about the current pin states and variable values.

void sendPinsStatus(){
  sendValue(pin0_tag, String(digitalRead(D0)));    // send the digital input D0 state
  sendValue(pin1_tag, String(digitalRead(D1)));    // send the digital input D1 state
  sendValue(pin5_tag, String(digitalRead(D5)));    // send the digital ouput D5 state

  // add more here like the next lines...
  // sendValue(pin6_tag, String(digitalRead(D6))); 
  // sendValue(pin8_tag, String(digitalRead(D8))); 
  }

//===================================================== onValueReceived
// It is called every time a new value received
  void onValueReceived(String tag, String value){
    Serial.println("Received: tag="+tag+ "  value="+value);
    if (tag== pin5_tag) {                    // write to digital pin D5
      int v=value.toInt();
      if (v==1) digitalWrite(D5,HIGH); else digitalWrite(D5,LOW);
    }
    
    // add more here like the next lines...
    /*
    if (tag==pin6_tag) {                    // write to digital Output D6
      int v=value.toInt();        //store the new PWM value to pin6_lastValue
      if (v==1) digitalWrite(D6,HIGH); else digitalWrite(6,LOW);
    }
    */
    
    }

//===================================================== setup
//=====================================================
  void setup() {
      Serial.begin(115200);
      while (!Serial) delay(1);
      pinMode(D5,OUTPUT);                 // on this example the pin D5 is used as OUTPUT
      // pinMode(D6,OUTPUT);              // add more Outputs here
      connectToWiFiNetwork();
      webSocket.begin();
      webSocket.onEvent(webSocketEvent);
  }

//===================================================== loop
//=====================================================
  void loop() {
      webSocket.loop();     
  
      //---Example: How to send a Digital Input state to Virtuino app every time it changes
      int pin0= digitalRead(D0);            // read the new state
      if (pin0_lastValue != pin0) {         // compare with the previous input state
        sendValue(pin0_tag, String(pin0));  // send the new state
        pin0_lastValue=pin0;                // copy the new state to variable pin0_lastValue 
      }


       //---Example: How to send a Digital Input state to Virtuino app every time it changes
      int pin1= digitalRead(D1);            // read the new state
      if (pin1_lastValue != pin1) {         // compare with the previous input state
        sendValue(pin1_tag, String(pin1));  // send the new state
        pin1_lastValue=pin1;                // copy the new state to variable pin1_lastValue 
      }
  
      //--- Be careful: Don't add a line like the next. It will send values to Virtuino app on each loop circle.
      // sendValue(pinD0_tag, digitalRead(D0));

      // avoid to use the void Delay() in your code. This causes communication delays
      // vDelay(1000);   // Use the vDelay() instead of Delay() or better none of them

      // add your code here...

      
  }









/*=====================================================
  ======================  UTILS =======================
  =====================================================
  You don't need to make changes to the code below
*/
//===================================================== connectToWiFiNetwork
void connectToWiFiNetwork(){
  Serial.println("Connecting to "+String(ssid));
  // If you don't want to config IP manually disable the next two lines and the disables IPAddress variables  at the top of the code
  //IPAddress subnet(255, 255, 255, 0);        // set subnet mask to match your network
  //WiFi.config(ip, gateway, subnet);          // If you don't want to config IP manually disable this line
  WiFi.mode(WIFI_STA);                         // Configure the module as station only.
  WiFi.begin(ssid, password);
   while (WiFi.status() != WL_CONNECTED) {
     delay(500);
     Serial.print(".");
    }
   Serial.println("\nWiFi connected");
   Serial.println(WiFi.localIP());              // Insert this IP on Virtuino IoT server settings
}

 
//===================================================== sendValue
// This function sends a value to a Tag. 
// The tag and the value are converted to a json message before sending 

bool sendValue(const char* tag, String value){
  String json = "{\"";
  json+=tag;
  json+="\":\"";
  json+=value;
  json+="\"}";
  Serial.println("Send: "+json);
  return webSocket.broadcastTXT(json);     // This function sends the message to all connected clients.
}

//===================================================== webSocketEvent
//This is the server handler. It receives all the messages

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.printf("[%u] Disconnected!\n", num);
            break;
        case WStype_CONNECTED:{ 
            IPAddress ip = webSocket.remoteIP(num);
            Serial.printf("[%u] New client connected - IP: %d.%d.%d.%d \n", num, ip[0], ip[1], ip[2], ip[3]);
            sendPinsStatus();         // send the initial pin and variable values to the connected clients
            break;
        }
        case WStype_TEXT:  // a new message received 
            Serial.printf("[%u] Received: %s\n", num, payload);
            //-- The incoming payload is a json message. The following code extracts the value from json without extra library
            String str = (char*)payload;
            int p1 = str.indexOf("{\"");
            if (p1==0) {
              int p2 = str.indexOf("\":");
              if (p2>p1) {
                String tag = str.substring(p1+2,p2);
                p1 = str.indexOf(":\"",p2);
                if (p1>0) {
                   p2 = str.lastIndexOf("\"}");  
                   if (p2>0){
                      String value = str.substring(p1+2,p2);
                      onValueReceived(tag,value);        
                    }
                  }
                }
            }
            break;
    }
}


//===================================================== vDelay
  void vDelay(int delayInMillis){long t=millis()+delayInMillis;while (millis()<t) webSocket.loop();}
